local Skada = Skada
--local L = LibStub("AceLocale-3.0"):GetLocale("Skada", false)

Skada:AddLoadableModule("Stagger", nil, function(Skada, L)
	if Skada.db.profile.modulesBlocked.Stagger then return end

	local mod = Skada:NewModule("Stagger")

      local STAGGER_TICK_COUNT = 20

      local data = {}
      local stagger = 
      {
            tickCount = 0,
            hasIronskin = false,
            hasFortifyingBrew = false,
      }

      local function printdebug(...)
            --print(...)
      end

      local function getPurifyPercent()
            local talentGroup = GetActiveSpecGroup()
            _, _, _, elusiveDanceSelected, _ = GetTalentInfo(7, 1, talentGroup)

            local purifyPercent = 0.5
            if elusiveDanceSelected then
                  purifyPercent = purifyPercent + 0.15
            end
            printdebug("Purify percent: "..purifyPercent)
            return purifyPercent
      end

      local function takeDamage(samount, sabsorbed, sschool)
            stagger.tickCount = 0
      end

	local function SpellDamage(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
		local spellId, spellName, spellSchool, samount, soverkill, sschool, sresisted, sblocked, sabsorbed, scritical, sglancing, scrushing, soffhand, _ = ...
            if dstGUID == UnitGUID("player") then
                  if spellId == 124255 then
                        data.staggerTaken = data.staggerTaken + samount
                        if data.maxTick < samount then
                              data.maxTick = samount
                        end

                        stagger.tickCount = stagger.tickCount + 1
                        printdebug("Tick "..samount.." ("..stagger.tickCount..")")
                  else
                        printdebug("SPELL HIT "..samount.." "..dstName)
                        takeDamage(samount, sabsorbed, spellSchool)
                  end
            end
	end

	local function SwingDamage(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
		local samount, soverkill, sschool, sresisted, sblocked, sabsorbed, scritical, sglancing, scrushing, soffhand, _ = ...
            if dstGUID == UnitGUID("player") then
                  printdebug("MELEE HIT "..samount.." "..dstName)
                  takeDamage(samount, sabsorbed, sschool)
            end
	end

	local function SpellCast(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
		local spellId, spellName, spellSchool = ...
            if srcGUID == UnitGUID("player") and spellId == 119582 then
                  local purifiedAmount = UnitStagger("player") * getPurifyPercent()
                  data.staggerPurified = data.staggerPurified + purifiedAmount

                  printdebug("Purified "..purifiedAmount)
            end
	end

	local function AuraChanged(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
		local spellId, spellName, spellSchool = ...
            if srcGUID == UnitGUID("player") then
                  --printdebug(eventtype.." "..spellName.." ("..spellId..")")

                  if spellId == 215479 then
                        if eventtype == "SPELL_AURA_APPLIED" then
                              stagger.hasIronskin = true
                              printdebug(spellName.." applied")
                        elseif eventtype == "SPELL_AURA_REMOVED" then
                              stagger.hasIronskin = false
                              printdebug(spellName.." removed")
                        end
                  elseif spellId == 120954 then
                        if eventtype == "SPELL_AURA_APPLIED" then
                              stagger.hasFortifyingBrew = true
                              printdebug(spellName.." applied")
                        elseif eventtype == "SPELL_AURA_REMOVED" then
                              stagger.hasFortifyingBrew = false
                              printdebug(spellName.." removed")
                        end
                  end
                  
                  -- if spellId == 124275 or spellId == 124274 or spellId == 124273 then -- stagger
            end
	end

	local function ClearData()
            data = 
            {
                  staggerTaken = 0, 
                  staggerPurified = 0, 
                  maxTick = 0,
            }
	end

	function mod:OnEnable()
		mod.metadata = {showspots = false, ordersort = true}
            ClearData()

		Skada:RegisterForCL(SpellDamage, 'SPELL_DAMAGE', {src_is_interesting = false, dst_is_interesting_nopets = false})
		Skada:RegisterForCL(SpellDamage, 'SPELL_PERIODIC_DAMAGE', {src_is_interesting = true, dst_is_interesting_nopets = false})
		Skada:RegisterForCL(SpellDamage, 'SPELL_BUILDING_DAMAGE', {src_is_interesting = true, dst_is_interesting_nopets = false})
		Skada:RegisterForCL(SwingDamage, 'SWING_DAMAGE', {dst_is_interesting_nopets = true, dst_is_interesting_nopets = false})
		Skada:RegisterForCL(SpellCast, 'SPELL_CAST_SUCCESS', {src_is_interesting = true, dst_is_interesting_nopets = false})

            Skada:RegisterForCL(AuraChanged, 'SPELL_AURA_APPLIED', {src_is_interesting = true, dst_is_interesting_nopets = false})
            Skada:RegisterForCL(AuraChanged, 'SPELL_AURA_REMOVED', {src_is_interesting = true, dst_is_interesting_nopets = false})

		Skada:AddMode(self, "Stagger")
	end

	function mod:OnDisable()
		Skada:RemoveMode(self)
	end

	local function setSummary(set)
		return data.staggered
	end

	function mod:GetSetSummary(set)
		return setSummary
	end
      
      function mod:Wipe(win)
            ClearData()
      end

	function mod:Update(win, set)
            local damageStaggered = data.staggerTaken + data.staggerPurified
            if damageStaggered == 0 then
                  return
            end

		local staggered = win.dataset[1] or {}
		win.dataset[1] = staggered

            staggered.id = "staggered"
            staggered.label = "Damage staggered"
            staggered.valuetext = Skada:FormatNumber(damageStaggered)
            staggered.value = damageStaggered

		local staggerTaken = win.dataset[2] or {}
		win.dataset[2] = staggerTaken

            staggerTaken.id = "staggerTaken"
            staggerTaken.label = "Taken"
            staggerTaken.valuetext = Skada:FormatNumber(data.staggerTaken)..(" (%02.1f%%)"):format(data.staggerTaken / damageStaggered * 100)
            staggerTaken.value = data.staggerTaken

		local staggerPurified = win.dataset[3] or {}
		win.dataset[3] = staggerPurified

            staggerPurified.id = "staggerPurified"
            staggerPurified.label = "Purified"
            staggerPurified.valuetext = Skada:FormatNumber(data.staggerPurified)..(" (%02.1f%%)"):format(data.staggerPurified / damageStaggered * 100)
            staggerPurified.value = data.staggerPurified

		local maxTick = win.dataset[4] or {}
		win.dataset[4] = maxTick

            maxTick.id = "maxTick"
            maxTick.label = "Max. tick"
            maxTick.valuetext = Skada:FormatNumber(data.maxTick)
            maxTick.value = data.maxTick

		win.metadata.maxvalue = damageStaggered
	end
end)

